package gregapi.block.multitileentity;

import static gregapi.data.CS.*;
import gregapi.block.multitileentity.IMultiTileEntity.IMTE_OnRegistration;
import gregapi.block.multitileentity.IMultiTileEntity.IMTE_OnRegistrationClient;
import gregapi.block.multitileentity.IMultiTileEntity.IMTE_OnRegistrationFirst;
import gregapi.block.multitileentity.IMultiTileEntity.IMTE_OnRegistrationFirstOfRegister;
import gregapi.block.multitileentity.IMultiTileEntity.IMTE_OnServerLoad;
import gregapi.block.multitileentity.IMultiTileEntity.IMTE_OnServerSave;
import gregapi.code.ArrayListNoNulls;
import gregapi.code.HashSetNoNulls;
import gregapi.code.ItemStackContainer;
import gregapi.code.ItemStackMap;
import gregapi.data.LH;
import gregapi.item.CreativeTab;
import gregapi.recipes.GT_ModHandler;
import gregapi.recipes.GT_ModHandler.RecipeBits;
import gregapi.recipes.Recipe.RecipeMap;
import gregapi.util.UT;

import java.io.File;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import net.minecraft.block.Block;
import net.minecraft.creativetab.CreativeTabs;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.tileentity.TileEntity;

/**
 * @author Gregorius Techneticies
 * 
 * Create yourself a new MultiTileEntity Registry in order to have your very own 32767 Sub IDs.
 * You can do with those IDs whatever you want since this automatically creates your personal Item and DOESN'T use any Items or Blocks of GregTech itself.
 * 
 * Whatever you do, DO NOT GET THE UTTERLY RETARDED IDEA OF ADDING YOUR MULTITILEENTITIES TO MY REGISTRY!!!
 * 
 * ================================================================================================================================================
 * The way this whole System works is very simple. The setTileEntity call can set the TileEntity of your choice at every Location you want.
 * If now the BlockContainer doesn't return a TileEntity, but instead the ItemBlock manually sets the TileEntity, you can have every single
 * TileEntity being placed at every Block you want. If that Block then is compatible with your TileEntity (via Interfaces and such) it can
 * easily make use of the TileEntity no matter which one it is.
 * 
 * "But what is with the Loading of those TileEntities? Don't they get deleted on startup?" You think? No they don't get deleted. Minecraft
 * can load every TileEntity just by a Name->Class Map (you know about that when you have ever created a TileEntity yourself), and the remaining
 * Stats can be saved inside the NBT of the TileEntity.
 * 
 * In the end I have a dynamic collection of Blocks to get the vanilla Materials and Sound Effects right, a Registry of TileEntities to be
 * attached to those Blocks via additional custom ItemBlocks to enable everything, and an automatic Network Handler.
 * 
 * The only thing needed to be done manually is something that transmits the Data from the Server to the Client to set the proper TileEntity there.
 * 
 * In order to do that, just send one of the 5 Packets (PacketSyncDataByteAndIDs, PacketSyncDataShortAndIDs, PacketSyncDataIntegerAndIDs, 
 * PacketSyncDataLongAndIDs or PacketSyncDataByteArrayAndIDs) for transmitting the ID to the Client with aID1 = getMultiTileEntityRegistryID() and
 * aID2 = getMultiTileEntityID(). The Byte/Short/Integer/Long/ByteArray can be used for transmitting other Data, such as a Facing to the Client.
 * ================================================================================================================================================
 */
public class MultiTileEntityRegistry {
	private static final HashMap<String, MultiTileEntityRegistry> NAMED_REGISTRIES = new HashMap<String, MultiTileEntityRegistry>();
	private static final ItemStackMap<ItemStackContainer, MultiTileEntityRegistry> REGISTRIES = new ItemStackMap<ItemStackContainer, MultiTileEntityRegistry>();
	private static final HashSetNoNulls<Class> sRegisteredTileEntities = new HashSetNoNulls<Class>();
	private final HashSetNoNulls<Class> mRegisteredTileEntities = new HashSetNoNulls<Class>();
	
	public HashMap<Short, CreativeTabs> mCreativeTabs = new HashMap<Short, CreativeTabs>();
	public Map<Short, MultiTileEntityClassContainer> mRegistry = new HashMap<Short, MultiTileEntityClassContainer>();
	public List<MultiTileEntityClassContainer> mRegistrations = new ArrayListNoNulls<MultiTileEntityClassContainer>();
	
	public final String mNameInternal;
	public final MultiTileEntityBlockInternal mBlock;
	
	/** 
	 * @param aNameInternal the internal Name of the Item. DO NOT START YOUR UNLOCALISED NAME WITH "gt."!!!
	 */
	public MultiTileEntityRegistry(String aNameInternal) {
		if (!GAPI.mStartedPreInit) throw new IllegalStateException("The MultiTileEntity Registry must be initialised at the Preload Phase and not before, because it relies on an ItemBlock being created!");
		if (GAPI.mStartedInit) throw new IllegalStateException("The MultiTileEntity Registry must be initialised at the Preload Phase and not later, because it relies on an ItemBlock being created!");
		mNameInternal = aNameInternal;
        mBlock = new MultiTileEntityBlockInternal(this);
        REGISTRIES.put(new ItemStackContainer(mBlock, 1, W), this);
        NAMED_REGISTRIES.put(mNameInternal, this);
	}
	
	/** Whatever you do, DO NOT GET THE UTTERLY RETARDED IDEA OF ADDING YOUR MULTITILEENTITIES TO MY OWN REGISTRY!!! Create your own instance! */
	public static MultiTileEntityRegistry getRegistry(int aRegistryID) {
		return REGISTRIES.get(new ItemStackContainer(Item.getItemById(aRegistryID), 1, W));
	}
	
	/** Whatever you do, DO NOT GET THE UTTERLY RETARDED IDEA OF ADDING YOUR MULTITILEENTITIES TO MY OWN REGISTRY!!! Create your own instance! */
	public static MultiTileEntityRegistry getRegistry(String aRegistryName) {
		return NAMED_REGISTRIES.get(aRegistryName);
	}
	
	/** Adds a new MultiTileEntity. It is highly recommended to do this in either the PreInit or the Init Phase. PostInit might not work well.*/
	public ItemStack add(String aLocalised, String aCategoricalName, int aID, int aCreativeTabID, Class<? extends TileEntity> aClass, int aBlockMetaData, int aStackSize, MultiTileEntityBlock aBlock, NBTTagCompound aParameters, Object... aRecipe) {
		return add(aLocalised, aCategoricalName, new MultiTileEntityClassContainer(aID, aCreativeTabID, aClass, aBlockMetaData, aStackSize, aBlock, aParameters), aRecipe);
	}
	
	/** Adds a new MultiTileEntity. It is highly recommended to do this in either the PreInit or the Init Phase. PostInit might not work well.*/
	public ItemStack add(String aLocalised, String aCategoricalName, MultiTileEntityClassContainer aClassContainer, Object... aRecipe) {
		if (aClassContainer == null || aClassContainer.mID == W || aClassContainer.mID < 0 || mRegistry.containsKey(aClassContainer.mID)) return null;
		LH.add(mNameInternal+"."+aClassContainer.mID+".name", aLocalised);
		mRegistry.put(aClassContainer.mID, aClassContainer);
		mRegistrations.add(aClassContainer);
		if (!mCreativeTabs.containsKey(aClassContainer.mCreativeTabID)) mCreativeTabs.put(aClassContainer.mCreativeTabID, new CreativeTab(mNameInternal+"."+aClassContainer.mCreativeTabID, aCategoricalName, Item.getItemFromBlock(mBlock), aClassContainer.mCreativeTabID));
		if (sRegisteredTileEntities.add(aClassContainer.mCanonicalTileEntity.getClass()) &&	aClassContainer.mCanonicalTileEntity instanceof IMTE_OnRegistrationFirst			) ((IMTE_OnRegistrationFirst			)aClassContainer.mCanonicalTileEntity).onRegistrationFirst				(this, aClassContainer.mID);
		if (mRegisteredTileEntities.add(aClassContainer.mCanonicalTileEntity.getClass()) &&	aClassContainer.mCanonicalTileEntity instanceof IMTE_OnRegistrationFirstOfRegister	) ((IMTE_OnRegistrationFirstOfRegister	)aClassContainer.mCanonicalTileEntity).onRegistrationFirstOfRegister	(this, aClassContainer.mID);
		if (																				aClassContainer.mCanonicalTileEntity instanceof IMTE_OnRegistration					) ((IMTE_OnRegistration					)aClassContainer.mCanonicalTileEntity).onRegistration					(this, aClassContainer.mID);
		if (CODE_CLIENT &&																	aClassContainer.mCanonicalTileEntity instanceof IMTE_OnRegistrationClient			) ((IMTE_OnRegistrationClient			)aClassContainer.mCanonicalTileEntity).onRegistrationClient				(this, aClassContainer.mID);
		if (aRecipe != null && aRecipe.length > 1) {
			if (aRecipe[0] instanceof Object[]) aRecipe = (Object[])aRecipe[0];
			if (aRecipe.length > 2) GT_ModHandler.addCraftingRecipe(getItem(aClassContainer.mID), RecipeBits.DEFAULT_REV_NCC, aRecipe);
		}
		// A simple special case to make it easier to add a Machine to Recipe Lists without having to worry about anything.
		String
		tRecipeMapName = aClassContainer.mParameters.getString(NBT_RECIPEMAP);
		if (UT.Code.stringValid(tRecipeMapName)) {RecipeMap tMap = RecipeMap.RECIPE_MAPS.get(tRecipeMapName); if (tMap != null) tMap.mRecipeMachineList.add(getItem(aClassContainer.mID));}
		tRecipeMapName = aClassContainer.mParameters.getString(NBT_FUELMAP);
		if (UT.Code.stringValid(tRecipeMapName)) {RecipeMap tMap = RecipeMap.RECIPE_MAPS.get(tRecipeMapName); if (tMap != null) tMap.mRecipeMachineList.add(getItem(aClassContainer.mID));}
		return getItem(aClassContainer.mID);
	}
	
	public ItemStack getItem(int aID) {return getItem(aID, 1, null);}
	public ItemStack getItem(int aID, NBTTagCompound aNBT) {return getItem(aID, 1, aNBT);}
	public ItemStack getItem(int aID, long aAmount) {return getItem(aID, aAmount, null);}
	
	public ItemStack getItem(int aID, long aAmount, NBTTagCompound aNBT) {
		ItemStack rStack = UT.Stacks.make(mBlock, (int)aAmount, aID);
		if (aNBT == null || aNBT.hasNoTags()) {
			aNBT = new NBTTagCompound();
			MultiTileEntityContainer tTileEntityContainer = getNewTileEntityContainer(aID, aNBT);
			if (tTileEntityContainer != null) ((IMultiTileEntity)tTileEntityContainer.mTileEntity).writeItemNBT(aNBT);
		}
		rStack.setTagCompound(aNBT.hasNoTags()?null:aNBT);
		return rStack;
	}
	
	public String getLocal(int aID) {return LH.get(mNameInternal+"."+aID+".name");}
	public TileEntity getNewTileEntity(int aID) {MultiTileEntityContainer tContainer = getNewTileEntityContainer(aID, null); return tContainer == null ? null : tContainer.mTileEntity;}
	public TileEntity getNewTileEntity(ItemStack aStack) {MultiTileEntityContainer tContainer = getNewTileEntityContainer(aStack); return tContainer == null ? null : tContainer.mTileEntity;}
	public MultiTileEntityContainer getNewTileEntityContainer(ItemStack aStack) {return getNewTileEntityContainer(UT.Stacks.meta(aStack), aStack.getTagCompound());}
	public MultiTileEntityClassContainer getClassContainer(int aID) {return mRegistry.get((short)aID);}
	public MultiTileEntityClassContainer getClassContainer(ItemStack aStack) {return mRegistry.get(UT.Stacks.meta(aStack));}
	
	public MultiTileEntityContainer getNewTileEntityContainer(int aID, NBTTagCompound aNBT) {
		MultiTileEntityClassContainer tClass = mRegistry.get((short)aID);
		if (tClass == null || tClass.mBlock == null) return null;
		MultiTileEntityContainer rContainer = new MultiTileEntityContainer((TileEntity)UT.Reflection.callConstructor(tClass.mClass, -1, null, true), tClass.mBlock, tClass.mBlockMetaData);
		if (rContainer.mTileEntity == null) return null;
		((IMultiTileEntity)rContainer.mTileEntity).initFromNBT(aNBT == null || aNBT.hasNoTags() ? tClass.mParameters : UT.NBT.fuseNBTs(aNBT, tClass.mParameters), (short)aID, (short)Block.getIdFromBlock(mBlock));
		return rContainer;
	}
	
	public static void onServerLoad(File aSaveLocation) {for (Class tClass : sRegisteredTileEntities) if (IMTE_OnServerLoad.class.isAssignableFrom(tClass)) try {((IMTE_OnServerLoad)tClass.newInstance()).onServerLoad(aSaveLocation);} catch (Throwable e) {e.printStackTrace(ERR);}}
	public static void onServerSave(File aSaveLocation) {for (Class tClass : sRegisteredTileEntities) if (IMTE_OnServerSave.class.isAssignableFrom(tClass)) try {((IMTE_OnServerSave)tClass.newInstance()).onServerSave(aSaveLocation);} catch (Throwable e) {e.printStackTrace(ERR);}}
}