package gregapi.load;

import static gregapi.data.CS.*;
import gregapi.data.CS.ArmorsGT;
import gregapi.data.*;
import gregapi.oredict.OreDictItemData;
import gregapi.oredict.OreDictManager;
import gregapi.util.OM;
import gregapi.util.UT;
import net.minecraft.init.Items;

/**
 * @author Gregorius Techneticies
 * 
 * Loads the ItemList (IL.java) for most Items (everything except the GT ones).
 * This happens during the InitPhase, or if GregTech is installed within the PreInitPhase, since GregTech loads after all the Mods mentioned here, unlike the API.
 */
public class LoaderItemList implements Runnable {
	@Override public String toString() {return "Item List Loader";}
	
	@Override
	public void run() {
        IL.Bottle_Empty							.set(UT.Stacks.make(Items.glass_bottle, 1, 0), new OreDictItemData(OP.bottle, MT.Empty));
    	OP.bottle.mContainerItem =					 UT.Stacks.make(Items.glass_bottle, 1, 0);
        
        IL.Dye_Bonemeal							.set(UT.Stacks.make(Items.dye, 1, 15));
        IL.Dye_SquidInk							.set(UT.Stacks.make(Items.dye, 1, 0));
        IL.Dye_Cactus							.set(UT.Stacks.make(Items.dye, 1, 2));
        IL.Dye_Cocoa							.set(UT.Stacks.make(Items.dye, 1, 3));
        
        IL.Food_Bread							.set(UT.Stacks.make(Items.bread, 1, 0));
        IL.Food_Carrot							.set(UT.Stacks.make(Items.carrot, 1, 0));
        IL.Food_Potato							.set(UT.Stacks.make(Items.potato, 1, 0));
        IL.Food_Potato_Baked					.set(UT.Stacks.make(Items.baked_potato, 1, 0));
        IL.Food_Potato_Poisonous				.set(UT.Stacks.make(Items.poisonous_potato, 1, 0));
        
        IL.TE_Cinnabar							.set(UT.Stacks.make(MD.TE_FOUNDATION, "material"						, 1,  20), new OreDictItemData(MT.OREMATS.Cinnabar, U));
        IL.TE_Slag								.set(UT.Stacks.make(MD.TE, "material"									, 1, 514));
        IL.TE_Slag_Rich							.set(UT.Stacks.make(MD.TE, "material"									, 1, 515));
        IL.TE_Phyto_Gro							.set(UT.Stacks.make(MD.TE, "material"									, 1, 516), null, "itemFertilizer");
        IL.TE_Phyto_Gro_Rich					.set(UT.Stacks.make(MD.TE, "material"									, 1, 517), null, "itemFertilizer");
        IL.TE_Rockwool							.set(UT.Stacks.make(MD.TE, "Rockwool"									, 1,   0));
        
        IL.RC_ShuntingWire						.set(UT.Stacks.make(MD.RC, "machine.delta"								, 1, 0));
        IL.RC_ShuntingWireFrame					.set(UT.Stacks.make(MD.RC, "frame"										, 1, 0));
        IL.RC_Post_Metal						.set(UT.Stacks.make(MD.RC, "post"										, 1, 2));
        IL.RC_Concrete							.set(UT.Stacks.make(MD.RC, "cube"										, 1, 1), new OreDictItemData(MT.Stone, 5*U/8));
        IL.RC_Crushed_Obsidian					.set(UT.Stacks.make(MD.RC, "cube"										, 1, 4), new OreDictItemData(MT.Obsidian, U*8));
        IL.RC_Stone_Abyssal						.set(UT.Stacks.make(MD.RC, "cube"										, 1, 6), OP.stone.dat(MT.Basalt));
        IL.RC_Stone_Quarried					.set(UT.Stacks.make(MD.RC, "cube"										, 1, 7), OP.stone.dat(MT.Marble));
        IL.RC_Creosote_Wood						.set(UT.Stacks.make(MD.RC, "cube"										, 1, 8), new OreDictItemData(MT.Wood, U*8));
        IL.RC_Rail_Standard						.set(UT.Stacks.make(MD.RC, "part.rail"									, 1, 0));
        IL.RC_Rail_Adv							.set(UT.Stacks.make(MD.RC, "part.rail"									, 1, 1));
        IL.RC_Rail_Wooden						.set(UT.Stacks.make(MD.RC, "part.rail"									, 1, 2));
        IL.RC_Rail_HS							.set(UT.Stacks.make(MD.RC, "part.rail"									, 1, 3));
        IL.RC_Rail_Reinforced					.set(UT.Stacks.make(MD.RC, "part.rail"									, 1, 4));
        IL.RC_Rail_Electric						.set(UT.Stacks.make(MD.RC, "part.rail"									, 1, 5));
        IL.RC_Tie_Wood							.set(UT.Stacks.make(MD.RC, "part.tie"									, 1, 0), new OreDictItemData(MT.Wood, 3*U/2));
        IL.RC_Tie_Stone							.set(UT.Stacks.make(MD.RC, "part.tie"									, 1, 1), new OreDictItemData(MT.Stone, 3*U/2));
        IL.RC_Bed_Wood							.set(UT.Stacks.make(MD.RC, "part.railbed"								, 1, 0), new OreDictItemData(MT.Wood, U*6));
        IL.RC_Bed_Stone							.set(UT.Stacks.make(MD.RC, "part.railbed"								, 1, 1), new OreDictItemData(MT.Stone, U*6));
        IL.RC_Rebar								.set(UT.Stacks.make(MD.RC, "part.rebar"									, 1, 0));
        
        IL.RC_Crowbar_Iron						.set(UT.Stacks.make(MD.RC, "tool.crowbar"								, 1, 0), new OreDictItemData(MT.Fe, U*3));
        IL.RC_Crowbar_Steel						.set(UT.Stacks.make(MD.RC, "tool.crowbar.reinforced"					, 1, 0), new OreDictItemData(MT.Steel, U*3));
        IL.RC_Crowbar_Thaumium					.set(UT.Stacks.make(MD.RC, "tool.crowbar.magic"							, 1, 0), new OreDictItemData(MT.Thaumium, U*3));
        IL.RC_Crowbar_Voidmetal					.set(UT.Stacks.make(MD.RC, "tool.crowbar.void"							, 1, 0), new OreDictItemData(MT.VoidMetal, U*3));
        
        IL.Tool_Sword_Steel						.set(UT.Stacks.make(MD.RC, "tool.steel.sword"							, 1, 0));
        IL.Tool_Pickaxe_Steel					.set(UT.Stacks.make(MD.RC, "tool.steel.pickaxe"							, 1, 0));
        IL.Tool_Shovel_Steel					.set(UT.Stacks.make(MD.RC, "tool.steel.shovel"							, 1, 0));
        IL.Tool_Axe_Steel						.set(UT.Stacks.make(MD.RC, "tool.steel.axe"								, 1, 0));
        IL.Tool_Hoe_Steel						.set(UT.Stacks.make(MD.RC, "tool.steel.hoe"								, 1, 0));
        
        IL.TF_LiveRoot							.set(UT.Stacks.make(MD.TF, "item.liveRoot"								, 1, 0), new OreDictItemData(MT.LiveRoot, U));
        IL.TF_Vial_FieryBlood					.set(UT.Stacks.make(MD.TF, "item.fieryBlood"							, 1, 0));
        IL.TF_Vial_FieryTears					.set(UT.Stacks.make(MD.TF, "item.fieryTears"							, 1, 0));
        IL.TF_Hydrachop_Raw						.set(UT.Stacks.make(MD.TF, "item.hydraChop"								, 1, 0), new OreDictItemData(MT.MeatRaw		, U*8, MT.Bone, U*2), "listAllhydraraw");
        IL.TF_Meef_Raw							.set(UT.Stacks.make(MD.TF, "item.meefRaw"								, 1, 0), new OreDictItemData(MT.MeatRaw		, U*2, MT.Bone, U/9), "listAllbeefraw");
        IL.TF_Meef_Cooked						.set(UT.Stacks.make(MD.TF, "item.meefSteak"								, 1, 0), new OreDictItemData(MT.MeatCooked	, U*2, MT.Bone, U/9), "listAllbeefcooked");
        IL.TF_Venison_Raw						.set(UT.Stacks.make(MD.TF, "item.venisonRaw"							, 1, 0), new OreDictItemData(MT.MeatRaw		, U*2, MT.Bone, U/9), "foodVenisonraw");
        IL.TF_Venison_Cooked					.set(UT.Stacks.make(MD.TF, "item.venisonCooked"							, 1, 0), new OreDictItemData(MT.MeatCooked	, U*2, MT.Bone, U/9), "foodVenisoncooked");
        IL.TF_Mushgloom							.set(UT.Stacks.make(MD.TF, "tile.TFPlant"								, 1, 9), new OreDictItemData(MT.Glowstone	, U/4), "listAllmushroom");
        
        if (IL.TF_Vial_FieryBlood.getItem() != null) IL.TF_Vial_FieryBlood.getItem().setContainerItem(Items.glass_bottle);
        if (IL.TF_Vial_FieryTears.getItem() != null) IL.TF_Vial_FieryTears.getItem().setContainerItem(Items.glass_bottle);
        
        IL.MoCr_Crab_Raw						.set(UT.Stacks.make(MD.MoCr, "crabraw"									, 1, 0), new OreDictItemData(MT.MeatRaw		, 3*U/2), "foodCrabraw");
        IL.MoCr_Crab_Cooked						.set(UT.Stacks.make(MD.MoCr, "crabcooked"								, 1, 0), new OreDictItemData(MT.MeatCooked	, 3*U/2), "foodCrabcooked");
        IL.MoCr_Turkey_Raw						.set(UT.Stacks.make(MD.MoCr, "turkeyraw"								, 1, 0), new OreDictItemData(MT.MeatRaw		, U*3, MT.Bone, U/4), "foodTurkeyraw");
        IL.MoCr_Turkey_Cooked					.set(UT.Stacks.make(MD.MoCr, "turkeycooked"								, 1, 0), new OreDictItemData(MT.MeatCooked	, U*3, MT.Bone, U/4), "foodTurkeycooked");
        IL.MoCr_Rat_Raw							.set(UT.Stacks.make(MD.MoCr, "ratraw"									, 1, 0), new OreDictItemData(MT.MeatRaw		, U*1, MT.Bone, U/9), "foodRatraw");
        IL.MoCr_Rat_Cooked						.set(UT.Stacks.make(MD.MoCr, "ratcooked"								, 1, 0), new OreDictItemData(MT.MeatCooked	, U*1, MT.Bone, U/9), "foodRatcooked");
        IL.MoCr_Ostrich_Raw						.set(UT.Stacks.make(MD.MoCr, "ostrichraw"								, 1, 0), new OreDictItemData(MT.MeatRaw		, U*2, MT.Bone, U/4), "foodOstrichraw");
        IL.MoCr_Ostrich_Cooked					.set(UT.Stacks.make(MD.MoCr, "ostrichcooked"							, 1, 0), new OreDictItemData(MT.MeatCooked	, U*2, MT.Bone, U/4), "foodOstrichcooked");
        IL.MoCr_Turtle_Raw						.set(UT.Stacks.make(MD.MoCr, "turtleraw"								, 1, 0), new OreDictItemData(MT.MeatRaw		, U*2, MT.Bone, U/9), "foodTurtleraw");
        
        IL.EBXL_Cactus_Paste					.set(UT.Stacks.make(MD.EBXL, "extrabiomes.paste"						, 1, 0));
        IL.EBXL_Dye_Black						.set(UT.Stacks.make(MD.EBXL, "extrabiomes.dye"							, 1, 0), null, DYE_OREDICTS_MIXABLE[DYE_INDEX_Black]);
        IL.EBXL_Dye_Blue						.set(UT.Stacks.make(MD.EBXL, "extrabiomes.dye"							, 1, 1), null, DYE_OREDICTS_MIXABLE[DYE_INDEX_Blue]);
        IL.EBXL_Dye_Brown						.set(UT.Stacks.make(MD.EBXL, "extrabiomes.dye"							, 1, 2), null, DYE_OREDICTS_MIXABLE[DYE_INDEX_Brown]);
        IL.EBXL_Dye_White						.set(UT.Stacks.make(MD.EBXL, "extrabiomes.dye"							, 1, 3), null, DYE_OREDICTS_MIXABLE[DYE_INDEX_White]);
        
        IL.BoP_Mudbrick							.set(UT.Stacks.make(MD.BoP, "misc"										, 1, 0));
        IL.BoP_Ashes							.set(UT.Stacks.make(MD.BoP, "misc"										, 1, 1), null, DYE_OREDICTS_MIXABLE[DYE_INDEX_Gray]);
        IL.BoP_Comb								.set(UT.Stacks.make(MD.BoP, "misc"										, 1, 2), null, "beeComb", "materialWaxcomb");
        IL.BoP_Flesh							.set(UT.Stacks.make(MD.BoP, "misc"										, 1, 3), new OreDictItemData(MT.MeatRaw, U));
        IL.BoP_Celestial						.set(UT.Stacks.make(MD.BoP, "misc"										, 1, 4));
        IL.BoP_Dye_Blue							.set(UT.Stacks.make(MD.BoP, "misc"										, 1, 5), null, DYE_OREDICTS_MIXABLE[DYE_INDEX_Blue]);
        IL.BoP_Dye_Brown						.set(UT.Stacks.make(MD.BoP, "misc"										, 1, 6), null, DYE_OREDICTS_MIXABLE[DYE_INDEX_Brown]);
        IL.BoP_Dye_Green						.set(UT.Stacks.make(MD.BoP, "misc"										, 1, 7), null, DYE_OREDICTS_MIXABLE[DYE_INDEX_Green]);
        IL.BoP_Dye_White						.set(UT.Stacks.make(MD.BoP, "misc"										, 1, 8), null, DYE_OREDICTS_MIXABLE[DYE_INDEX_White]);
        IL.BoP_Dye_Black						.set(UT.Stacks.make(MD.BoP, "misc"										, 1, 9), null, DYE_OREDICTS_MIXABLE[DYE_INDEX_Black]);
        IL.BoP_GhastlySoul						.set(UT.Stacks.make(MD.BoP, "misc"										, 1,10));
        IL.BoP_PixieDust						.set(UT.Stacks.make(MD.BoP, "misc"										, 1,11));
        IL.BoP_Ichor							.set(UT.Stacks.make(MD.BoP, "misc"										, 1,12));
        IL.BoP_Pinecone							.set(UT.Stacks.make(MD.BoP, "misc"										, 1,13));
        
        IL.BoP_Berry							.set(UT.Stacks.make(MD.BoP, "food"										, 1, 0), null, "listAllberry", "listAllfruit");
        IL.BoP_ShroomPowder						.set(UT.Stacks.make(MD.BoP, "food"										, 1, 1));
        IL.BoP_WildCarrots						.set(UT.Stacks.make(MD.BoP, "food"										, 1, 2), null, "cropWildcarrots");
        IL.BoP_Peach							.set(UT.Stacks.make(MD.BoP, "food"										, 1, 3), null, "cropPeach");
        IL.BoP_Persimmon						.set(UT.Stacks.make(MD.BoP, "food"										, 1, 8), null, "cropPersimmon");
        IL.BoP_HoneyComb						.set(UT.Stacks.make(MD.BoP, "food"										, 1, 9), null, "beeComb", "materialHoneycomb", "foodFilledhoneycomb");
        IL.BoP_Ambrosia							.set(UT.Stacks.make(MD.BoP, "food"										, 1,10), null, "foodAmbrosia");
        IL.BoP_Turnip							.set(UT.Stacks.make(MD.BoP, "food"										, 1,11), null, "cropTurnip");
        IL.BoP_Pear								.set(UT.Stacks.make(MD.BoP, "food"										, 1,12), null, "cropPear");
        
        IL.BoP_Jar_Empty						.set(UT.Stacks.make(MD.BoP, "jarEmpty"									, 1, 0), new OreDictItemData(MT.Glass, 7*U/3));
        IL.BoP_Jar_Honey						.set(UT.Stacks.make(MD.BoP, "jarFilled"									, 1, 0), new OreDictItemData(MT.Glass, 7*U/3), "foodHoneydrop");
        IL.BoP_Jar_Poison						.set(UT.Stacks.make(MD.BoP, "jarFilled"									, 1, 1), new OreDictItemData(MT.Glass, 7*U/3));
        IL.BoP_Jar_Pixie						.set(UT.Stacks.make(MD.BoP, "jarFilled"									, 1, 2), new OreDictItemData(MT.Glass, 7*U/3));
        
        if (IL.BoP_Jar_Honey.getItem() != null && IL.BoP_Jar_Empty.getItem() != null) IL.BoP_Jar_Honey.getItem().setContainerItem(IL.BoP_Jar_Empty.getItem());
        
        IL.RoC_Comb_Slippery					.set(UT.Stacks.make(MD.RoC, "rotarycraft_item_modinterface"				, 1, 0), null, "beeComb");
        IL.RoC_Propolis_Slippery				.set(UT.Stacks.make(MD.RoC, "rotarycraft_item_modinterface"				, 1, 1), null, "listAllpropolis");
        
        IL.JABBA_Dolly							.set(UT.Stacks.make(MD.JABBA, "mover"									, 1, 0));
        IL.JABBA_Dolly_Diamond					.set(UT.Stacks.make(MD.JABBA, "moverDiamond"							, 1, 0));
        
        IL.MFR_Fertilizer						.set(UT.Stacks.make(MD.MFR, "fertilizer"								, 1, 0), null, "itemFertilizer", DYE_OREDICTS_MIXABLE[DYE_INDEX_Brown]);
        
        IL.FR_Planks_Fireproof					.set(UT.Stacks.make(MD.FR, "planksFireproof"							, 1, 0));
        IL.FR_Planks							.set(UT.Stacks.make(MD.FR, "planks"										, 1, 0));
        IL.FR_Slabs_Fireproof					.set(UT.Stacks.make(MD.FR, "slabsFireproof"								, 1, 0));
        IL.FR_Slabs								.set(UT.Stacks.make(MD.FR, "slabs"										, 1, 0));
        IL.FR_Logs_Fireproof					.set(UT.Stacks.make(MD.FR, "logsFireproof"								, 1, 0));
        IL.FR_Logs								.set(UT.Stacks.make(MD.FR, "logs"										, 1, 0));
        IL.FR_Phosphor							.set(UT.Stacks.make(MD.FR, "phosphor"									, 1, 0), new OreDictItemData(MT.P, U));
        IL.FR_Royal_Jelly						.set(UT.Stacks.make(MD.FR, "royalJelly"									, 1, 0));
        IL.FR_Propolis							.set(UT.Stacks.make(MD.FR, "propolis"									, 1, 0), null, "listAllpropolis");
        IL.FR_Propolis_Sticky					.set(UT.Stacks.make(MD.FR, "propolis"									, 1, 1), null, "listAllpropolis");
        IL.FR_Propolis_Pulsating				.set(UT.Stacks.make(MD.FR, "propolis"									, 1, 2), null, "listAllpropolis");
        IL.FR_Propolis_Silky					.set(UT.Stacks.make(MD.FR, "propolis"									, 1, 3), null, "listAllpropolis");
        IL.FR_Comb_Honey						.set(UT.Stacks.make(MD.FR, "beeCombs"									, 1, 0), null, "beeComb", "materialHoneycomb", "foodFilledhoneycomb");
        IL.FR_Comb_Cocoa						.set(UT.Stacks.make(MD.FR, "beeCombs"									, 1, 1), null, "beeComb");
        IL.FR_Comb_Simmering					.set(UT.Stacks.make(MD.FR, "beeCombs"									, 1, 2), null, "beeComb");
        IL.FR_Comb_Stringy						.set(UT.Stacks.make(MD.FR, "beeCombs"									, 1, 3), null, "beeComb");
        IL.FR_Comb_Frozen						.set(UT.Stacks.make(MD.FR, "beeCombs"									, 1, 4), null, "beeComb");
        IL.FR_Comb_Dripping						.set(UT.Stacks.make(MD.FR, "beeCombs"									, 1, 5), null, "beeComb");
        IL.FR_Comb_Silky						.set(UT.Stacks.make(MD.FR, "beeCombs"									, 1, 6), null, "beeComb");
        IL.FR_Comb_Parched						.set(UT.Stacks.make(MD.FR, "beeCombs"									, 1, 7), null, "beeComb");
        IL.FR_Comb_Mysterious					.set(UT.Stacks.make(MD.FR, "beeCombs"									, 1, 8), null, "beeComb");
        IL.FR_Comb_Irradiated					.set(UT.Stacks.make(MD.FR, "beeCombs"									, 1, 9), null, "beeComb");
        IL.FR_Comb_Powdery						.set(UT.Stacks.make(MD.FR, "beeCombs"									, 1,10), null, "beeComb");
        IL.FR_Comb_Reddened						.set(UT.Stacks.make(MD.FR, "beeCombs"									, 1,11), null, "beeComb");
        IL.FR_Comb_Darkened						.set(UT.Stacks.make(MD.FR, "beeCombs"									, 1,12), null, "beeComb");
        IL.FR_Comb_Omega						.set(UT.Stacks.make(MD.FR, "beeCombs"									, 1,13), null, "beeComb");
        IL.FR_Comb_Wheaten						.set(UT.Stacks.make(MD.FR, "beeCombs"									, 1,14), null, "beeComb");
        IL.FR_Comb_Mossy						.set(UT.Stacks.make(MD.FR, "beeCombs"									, 1,15), null, "beeComb");
        IL.FR_Comb_Mellow						.set(UT.Stacks.make(MD.FR, "beeCombs"									, 1,16), null, "beeComb");
        IL.FR_Pollen_Cluster					.set(UT.Stacks.make(MD.FR, "pollen"										, 1, 0));
        IL.FR_Pollen_Cluster_Crystalline		.set(UT.Stacks.make(MD.FR, "pollen"										, 1, 1));
        IL.FR_Mulch								.set(UT.Stacks.make(MD.FR, "mulch"										, 1, 0));
        IL.FR_Fertilizer						.set(UT.Stacks.make(MD.FR, "fertilizerCompound"							, 1, 0), null, "itemFertilizer");
        IL.FR_Compost							.set(UT.Stacks.make(MD.FR, "fertilizerBio"								, 1, 0));
        IL.FR_Pulsating_Dust					.set(UT.Stacks.make(MD.FR, "craftingMaterial"							, 1, 0), new OreDictItemData(MT.EnderPearl, U));
        IL.FR_Pulsating_Mesh					.set(UT.Stacks.make(MD.FR, "craftingMaterial"							, 1, 1));
        IL.FR_Silk								.set(UT.Stacks.make(MD.FR, "craftingMaterial"							, 1, 2));
        IL.FR_Silk_Woven						.set(UT.Stacks.make(MD.FR, "craftingMaterial"							, 1, 3));
        IL.FR_Dissipation_Capsule				.set(UT.Stacks.make(MD.FR, "craftingMaterial"							, 1, 4));
        IL.FR_Ice_Shard							.set(UT.Stacks.make(MD.FR, "craftingMaterial"							, 1, 5), new OreDictItemData(MT.Ice, U));
        IL.FR_Iodine_Capsule					.set(UT.Stacks.make(MD.FR, "iodineCapsule"								, 1, 0));
        IL.FR_HoneyPot							.set(UT.Stacks.make(MD.FR, "honeyPot"									, 1, 0));
        IL.FR_Ambrosia							.set(UT.Stacks.make(MD.FR, "ambrosia"									, 1, 0), null, "foodAmbrosia");
        IL.FR_TinCapsule						.set(UT.Stacks.make(MD.FR, "canEmpty"									, 1, 0));
        IL.FR_WaxCapsule						.set(UT.Stacks.make(MD.FR, "waxCapsule"									, 1, 0));
        IL.FR_RefractoryCapsule					.set(UT.Stacks.make(MD.FR, "refractoryEmpty"							, 1, 0));
        IL.FR_Candle							.set(UT.Stacks.make(MD.FR, "candle"										, 1, 0));
        IL.FR_Bee_Drone							.set(UT.Stacks.make(MD.FR, "beeDroneGE"									, 1, 0));
        IL.FR_Bee_Princess						.set(UT.Stacks.make(MD.FR, "beePrincessGE"								, 1, 0));
        IL.FR_Bee_Queen							.set(UT.Stacks.make(MD.FR, "beeQueenGE"									, 1, 0));
        IL.FR_Tree_Sapling						.set(UT.Stacks.make(MD.FR, "sapling"									, 1, UT.Stacks.make(MD.FR, "saplingGE", 1)));
        IL.FR_Butterfly							.set(UT.Stacks.make(MD.FR, "butterflyGE"								, 1, 0));
        IL.FR_Larvae							.set(UT.Stacks.make(MD.FR, "beeLarvaeGE"								, 1, 0));
        IL.FR_Serum								.set(UT.Stacks.make(MD.FR, "serumGE"									, 1, 0));
        IL.FR_Caterpillar						.set(UT.Stacks.make(MD.FR, "caterpillarGE"								, 1, 0));
        IL.FR_PollenFertile						.set(UT.Stacks.make(MD.FR, "pollenFertile"								, 1, 0));
        IL.FR_Stick								.set(UT.Stacks.make(MD.FR, "oakStick"									, 1, 0));
        IL.FR_Casing_Impregnated				.set(UT.Stacks.make(MD.FR, "impregnatedCasing"							, 1, 0));
        IL.FR_Casing_Sturdy						.set(UT.Stacks.make(MD.FR, "sturdyMachine"								, 1, 0));
        IL.FR_Casing_Hardened					.set(UT.Stacks.make(MD.FR, "hardenedMachine"							, 1, 0));
    	
        IL.Cell_Universal_Fluid					.set(UT.Stacks.makeIC2("FluidCell"											, 1   ), new OreDictItemData(MT.Sn, U*2, OM.stack(MT.Glass, (U * 3)/ 8)));
        IL.Cell_Empty							.set(UT.Stacks.makeIC2("cell"												, 1   ), OP.cell.dat(MT.Empty));
        IL.Cell_Water							.set(UT.Stacks.makeIC2("waterCell"											, 1   ), OP.cell.dat(MT.Water));
        IL.Cell_Lava							.set(UT.Stacks.makeIC2("lavaCell"											, 1   ), OP.cell.dat(MT.Lava));
        IL.Cell_Air								.set(UT.Stacks.makeIC2("airCell"											, 1   ), OP.cell.dat(MT.Air));
        IL.Cell_CFoam							.set(UT.Stacks.makeIC2("CFCell"												, 1   ), OP.cell.dat(MT.ConstructionFoam));
        IL.Cell_UUM								.set(UT.Stacks.makeIC2("uuMatterCell"										, 1   ), OP.cell.dat(MT.UUMatter));
    	OP.cell.mContainerItem =					 UT.Stacks.makeIC2("cell"												, 1   , OreDictManager.INSTANCE.getFirstOre("cellEmpty", 1));
        
    	IL.IC2_Debug							.set(UT.Stacks.makeIC2("debug"												, 1   ));
        IL.IC2_Scaffold							.set(UT.Stacks.makeIC2("scaffold"											, 1   ), new OreDictItemData(MT.Wood, 9*U/8));
        IL.IC2_Scaffold_Iron					.set(UT.Stacks.makeIC2("ironScaffold"										, 1   ), new OreDictItemData(MT.Fe, 15*U/32));
        IL.IC2_Foam								.set(UT.Stacks.makeIC2("constructionFoam"									, 1   ));
        IL.IC2_Foam_Reinforced					.set(UT.Stacks.makeIC2("constructionreinforcedFoam"							, 1   ));
        IL.IC2_Wall								.set(UT.Stacks.makeIC2("constructionFoamWall"								, 1   ));
        IL.IC2_Wall_Reinforced					.set(UT.Stacks.makeIC2("reinforcedStone"									, 1   ));
        IL.IC2_Spray_WeedEx						.set(UT.Stacks.makeIC2("weedEx"												, 1   ), new OreDictItemData(MT.Sn, U, MT.Redstone, U));
        IL.IC2_Mixed_Metal_Ingot				.set(UT.Stacks.makeIC2("mixedMetalIngot"									, 1   ));
        IL.IC2_Fertilizer						.set(UT.Stacks.makeIC2("fertilizer"											, 1   ), null, "itemFertilizer");
        IL.IC2_Resin							.set(UT.Stacks.makeIC2("resin"												, 1   ));
        IL.IC2_Log_Rubber						.set(UT.Stacks.makeIC2("rubberWood"											, 1   ));
        IL.IC2_Leaves_Rubber					.set(UT.Stacks.makeIC2("rubberLeaves"										, 1   ));
        IL.IC2_Sapling_Rubber					.set(UT.Stacks.makeIC2("rubberSapling"										, 1   ));
    	IL.IC2_Plantball						.set(UT.Stacks.makeIC2("plantBall"											, 1   ));
    	IL.IC2_Biochaff							.set(UT.Stacks.makeIC2("biochaff"											, 1   , IL.IC2_Plantball.get(1)));
    	IL.IC2_Crop_Seeds						.set(UT.Stacks.makeIC2("cropSeed"											, 1   ));
    	IL.IC2_Grin_Powder						.set(UT.Stacks.makeIC2("grinPowder"											, 1   ));
    	IL.IC2_Energium_Dust					.set(UT.Stacks.makeIC2("energiumDust"										, 1   ));
        IL.IC2_Scrap							.set(UT.Stacks.makeIC2("scrap"												, 1   ));
        IL.IC2_Scrapbox							.set(UT.Stacks.makeIC2("scrapBox"											, 1   ));
        IL.IC2_Fuel_Rod_Empty					.set(UT.Stacks.makeIC2("fuelRod"											, 1   ), new OreDictItemData(MT.Fe, U));
        IL.IC2_Food_Can_Empty					.set(UT.Stacks.makeIC2("tinCan"												, 1   ), new OreDictItemData(MT.Sn, U/2));
        IL.IC2_Food_Can_Filled					.set(UT.Stacks.makeIC2("filledTinCan"										, 1, 0));
        IL.IC2_Food_Can_Spoiled					.set(UT.Stacks.makeIC2("filledTinCan"										, 1, 1));
        IL.IC2_Industrial_Diamond				.set(UT.Stacks.makeIC2("industrialDiamond"									, 1, 0), OP.gem.dat(MT.TECH.DiamondIndustrial));
        IL.IC2_Coal_Ball						.set(UT.Stacks.makeIC2("coalBall"											, 1   ), new OreDictItemData(MT.Coal, U*8));
        IL.IC2_Compressed_Coal_Ball				.set(UT.Stacks.makeIC2("compressedCoalBall"									, 1   ), new OreDictItemData(MT.Coal, U*8));
        IL.IC2_Compressed_Coal_Chunk			.set(UT.Stacks.makeIC2("coalChunk"											, 1   ), new OreDictItemData(MT.Coal, U*64, MT.Obsidian, U*9));
        IL.IC2_Carbon_Fiber						.set(UT.Stacks.makeIC2("carbonFiber"										, 1   ));
        IL.IC2_Carbon_Mesh						.set(UT.Stacks.makeIC2("carbonMesh"											, 1   ));
        IL.IC2_Carbon_Plate						.set(UT.Stacks.makeIC2("carbonPlate"										, 1   ));
        IL.IC2_Advanced_Alloy					.set(UT.Stacks.makeIC2("advancedAlloy"										, 1   ));
        IL.IC2_ShaftIron						.set(UT.Stacks.makeIC2("ironshaft"											, 1   ), new OreDictItemData(MT.Fe, U*9));
        IL.IC2_ShaftSteel						.set(UT.Stacks.makeIC2("steelshaft"											, 1   ), new OreDictItemData(MT.Steel, U*9));
        IL.IC2_Machine							.set(UT.Stacks.makeIC2("machine"											, 1   ), new OreDictItemData(MT.Fe, U*8));
        
        IL.IC2_SuBattery						.set(UT.Stacks.makeIC2("suBattery"											, 1   ));
        IL.IC2_ReBattery						.set(UT.Stacks.makeIC2("reBattery"											, 1   ));
        IL.IC2_AdvBattery						.set(UT.Stacks.makeIC2("advBattery"											, 1   ));
        IL.IC2_EnergyCrystal					.set(UT.Stacks.makeIC2("energyCrystal"										, 1   ));
        IL.IC2_LapotronCrystal					.set(UT.Stacks.makeIC2("lapotronCrystal"									, 1   ));
        
        IL.Tool_Sword_Bronze					.set(UT.Stacks.makeIC2("bronzeSword"										, 1   ));
        IL.Tool_Pickaxe_Bronze					.set(UT.Stacks.makeIC2("bronzePickaxe"										, 1   ));
        IL.Tool_Shovel_Bronze					.set(UT.Stacks.makeIC2("bronzeShovel"										, 1   ));
        IL.Tool_Axe_Bronze						.set(UT.Stacks.makeIC2("bronzeAxe"											, 1   ));
        IL.Tool_Hoe_Bronze						.set(UT.Stacks.makeIC2("bronzeHoe"											, 1   ));
        IL.IC2_ForgeHammer						.set(UT.Stacks.makeIC2("ForgeHammer"										, 1   ));
        IL.IC2_WireCutter						.set(UT.Stacks.makeIC2("cutter"												, 1   ));
        
        IL.Credit_Iron							.set(UT.Stacks.makeIC2("coin"												, 1   ));
        
        IL.Upgrade_Overclocker					.set(UT.Stacks.makeIC2("overclockerUpgrade"									, 1   ));
        IL.Upgrade_Battery						.set(UT.Stacks.makeIC2("energyStorageUpgrade"								, 1   ));
        
        ArmorsGT.HAZMATS_FROST					.add(UT.Stacks.makeIC2("hazmatHelmet"										, 1, W));
        ArmorsGT.HAZMATS_FROST					.add(UT.Stacks.makeIC2("hazmatChestplate"									, 1, W));
        ArmorsGT.HAZMATS_FROST					.add(UT.Stacks.makeIC2("hazmatLeggings"										, 1, W));
        ArmorsGT.HAZMATS_FROST					.add(UT.Stacks.makeIC2("hazmatBoots"										, 1, W));
        ArmorsGT.HAZMATS_FROST					.add(UT.Stacks.make(MD.TF, "item.arcticHelm"							, 1, W));
        ArmorsGT.HAZMATS_FROST					.add(UT.Stacks.make(MD.TF, "item.arcticPlate"							, 1, W));
        ArmorsGT.HAZMATS_FROST					.add(UT.Stacks.make(MD.TF, "item.arcticLegs"							, 1, W));
        ArmorsGT.HAZMATS_FROST					.add(UT.Stacks.make(MD.TF, "item.arcticBoots"							, 1, W));
        ArmorsGT.HAZMATS_FROST					.add(UT.Stacks.make(MD.TF, "item.yetiHelm"								, 1, W));
        ArmorsGT.HAZMATS_FROST					.add(UT.Stacks.make(MD.TF, "item.yetiPlate"								, 1, W));
        ArmorsGT.HAZMATS_FROST					.add(UT.Stacks.make(MD.TF, "item.yetiLegs"								, 1, W));
        ArmorsGT.HAZMATS_FROST					.add(UT.Stacks.make(MD.TF, "item.yetiBoots"								, 1, W));
        
        ArmorsGT.HAZMATS_HEAT					.add(UT.Stacks.makeIC2("hazmatHelmet"										, 1, W));
        ArmorsGT.HAZMATS_HEAT					.add(UT.Stacks.makeIC2("hazmatChestplate"									, 1, W));
        ArmorsGT.HAZMATS_HEAT					.add(UT.Stacks.makeIC2("hazmatLeggings"										, 1, W));
        ArmorsGT.HAZMATS_HEAT					.add(UT.Stacks.makeIC2("hazmatBoots"										, 1, W));
        
        ArmorsGT.HAZMATS_INSECTS				.add(UT.Stacks.makeIC2("hazmatHelmet"										, 1, W));
        ArmorsGT.HAZMATS_INSECTS				.add(UT.Stacks.makeIC2("hazmatChestplate"									, 1, W));
        ArmorsGT.HAZMATS_INSECTS				.add(UT.Stacks.makeIC2("hazmatLeggings"										, 1, W));
        ArmorsGT.HAZMATS_INSECTS				.add(UT.Stacks.makeIC2("hazmatBoots"										, 1, W));
        ArmorsGT.HAZMATS_INSECTS				.add(UT.Stacks.make(MD.FR, "apiaristHelmet"								, 1, W));
        ArmorsGT.HAZMATS_INSECTS				.add(UT.Stacks.make(MD.FR, "apiaristChest"								, 1, W));
        ArmorsGT.HAZMATS_INSECTS				.add(UT.Stacks.make(MD.FR, "apiaristLegs"								, 1, W));
        ArmorsGT.HAZMATS_INSECTS				.add(UT.Stacks.make(MD.FR, "apiaristBoots"								, 1, W));
        
        ArmorsGT.HAZMATS_BIO					.add(UT.Stacks.makeIC2("hazmatHelmet"										, 1, W));
        ArmorsGT.HAZMATS_BIO					.add(UT.Stacks.makeIC2("hazmatChestplate"									, 1, W));
        ArmorsGT.HAZMATS_BIO					.add(UT.Stacks.makeIC2("hazmatLeggings"										, 1, W));
        ArmorsGT.HAZMATS_BIO					.add(UT.Stacks.makeIC2("hazmatBoots"										, 1, W));
        
        ArmorsGT.HAZMATS_GAS					.add(UT.Stacks.makeIC2("hazmatHelmet"										, 1, W));
        ArmorsGT.HAZMATS_GAS					.add(UT.Stacks.makeIC2("hazmatChestplate"									, 1, W));
        ArmorsGT.HAZMATS_GAS					.add(UT.Stacks.makeIC2("hazmatLeggings"										, 1, W));
        ArmorsGT.HAZMATS_GAS					.add(UT.Stacks.makeIC2("hazmatBoots"										, 1, W));
        
        ArmorsGT.HAZMATS_RADIOACTIVE			.add(UT.Stacks.makeIC2("hazmatHelmet"										, 1, W));
        ArmorsGT.HAZMATS_RADIOACTIVE			.add(UT.Stacks.makeIC2("hazmatChestplate"									, 1, W));
        ArmorsGT.HAZMATS_RADIOACTIVE			.add(UT.Stacks.makeIC2("hazmatLeggings"										, 1, W));
        ArmorsGT.HAZMATS_RADIOACTIVE			.add(UT.Stacks.makeIC2("hazmatBoots"										, 1, W));
        ArmorsGT.HAZMATS_RADIOACTIVE			.add(UT.Stacks.make(MD.ReC, "reactorcraft_item_hazhelmet"				, 1, W));
        ArmorsGT.HAZMATS_RADIOACTIVE			.add(UT.Stacks.make(MD.ReC, "reactorcraft_item_hazchest"				, 1, W));
        ArmorsGT.HAZMATS_RADIOACTIVE			.add(UT.Stacks.make(MD.ReC, "reactorcraft_item_hazlegs"					, 1, W));
        ArmorsGT.HAZMATS_RADIOACTIVE			.add(UT.Stacks.make(MD.ReC, "reactorcraft_item_hazboots"				, 1, W));
        
        ArmorsGT.HAZMATS_LIGHTNING				.add(UT.Stacks.makeIC2("hazmatHelmet"										, 1, W));
        ArmorsGT.HAZMATS_LIGHTNING				.add(UT.Stacks.makeIC2("hazmatChestplate"									, 1, W));
        ArmorsGT.HAZMATS_LIGHTNING				.add(UT.Stacks.makeIC2("hazmatLeggings"										, 1, W));
        ArmorsGT.HAZMATS_LIGHTNING				.add(UT.Stacks.makeIC2("hazmatBoots"										, 1, W));
        ArmorsGT.HAZMATS_LIGHTNING				.add(UT.Stacks.make(Items.chainmail_helmet									, 1, W));
        ArmorsGT.HAZMATS_LIGHTNING				.add(UT.Stacks.make(Items.chainmail_chestplate								, 1, W));
        ArmorsGT.HAZMATS_LIGHTNING				.add(UT.Stacks.make(Items.chainmail_leggings								, 1, W));
        ArmorsGT.HAZMATS_LIGHTNING				.add(UT.Stacks.make(Items.chainmail_boots									, 1, W));
	}
}