package gregapi.tileentity.delegate;

import static gregapi.data.CS.*;
import gregapi.random.IHasCoords;
import gregapi.random.IHasWorld;
import gregapi.util.UT;
import net.minecraft.block.Block;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.ChunkCoordinates;
import net.minecraft.world.World;
import net.minecraft.world.biome.BiomeGenBase;
import net.minecraftforge.common.util.ForgeDirection;

/**
 * @author Gregorius Techneticies
 */
public final class DelegatorTileEntity<T> implements IHasWorld, IHasCoords {
    /** the TileEntity. This should be an instance of TileEntity. */
	public final T mTileEntity;
	/** the Coordinates the TileEntity has, or should have in case mTileEntity is null. */
	public final int mX, mY, mZ;
	/** the World Object the TileEntity has, or should have in case mTileEntity is null. */
	public final World mWorld;
    /** the Side of the Delegate responsible for handling. So a TE-Tesseract alike can go a curve. */
	public final byte mSideOfTileEntity;

	public DelegatorTileEntity(DelegatorTileEntity<T> aDelegator) {
		mTileEntity = aDelegator.mTileEntity;
		mWorld = aDelegator.mWorld;
		mX = aDelegator.mX;
		mY = aDelegator.mY;
		mZ = aDelegator.mZ;
		mSideOfTileEntity = aDelegator.mSideOfTileEntity;
	}
	
	public DelegatorTileEntity(T aTileEntity, byte aSideOfTileEntity) {
		mTileEntity = aTileEntity;
		mSideOfTileEntity = aSideOfTileEntity;
		if (aTileEntity instanceof TileEntity) {
			mWorld = ((TileEntity)aTileEntity).getWorldObj();
			mX = ((TileEntity)aTileEntity).xCoord;
			mY = ((TileEntity)aTileEntity).yCoord;
			mZ = ((TileEntity)aTileEntity).zCoord;
		} else {
			throw new IllegalArgumentException("Parameter must be an instance of TileEntity and != null");
		}
	}
	
	public DelegatorTileEntity(T aTileEntity, DelegatorTileEntity aDelegator) {
		mTileEntity = aTileEntity;
		mWorld = aDelegator.mWorld;
		mX = aDelegator.mX;
		mY = aDelegator.mY;
		mZ = aDelegator.mZ;
		mSideOfTileEntity = aDelegator.mSideOfTileEntity;
	}
	
	public DelegatorTileEntity(T aTileEntity, World aWorld, int aX, int aY, int aZ, byte aSideOfTileEntity) {
		mTileEntity = aTileEntity;
		mWorld = aWorld;
		mX = aX;
		mY = aY;
		mZ = aZ;
		mSideOfTileEntity = aSideOfTileEntity;
	}
	
	public DelegatorTileEntity(T aTileEntity, World aWorld, ChunkCoordinates aCoords, byte aSideOfTileEntity) {
		mTileEntity = aTileEntity;
		mWorld = aWorld;
		mX = aCoords.posX;
		mY = aCoords.posY;
		mZ = aCoords.posZ;
		mSideOfTileEntity = aSideOfTileEntity;
	}
	
	public ForgeDirection getForgeSideOfTileEntity() {return FORGE_DIR[mSideOfTileEntity];}
	public Block getBlock() {return mWorld.getBlock(mX, mY, mZ);}
	public byte getMetaData() {return UT.Code.bind4(mWorld.getBlockMetadata(mX, mY, mZ));}
	public boolean setBlock(Block aBlock) {return mWorld.setBlock(mX, mY, mZ, aBlock, 0, 3);}
	public boolean setBlock(Block aBlock, int aMetaData) {return mWorld.setBlock(mX, mY, mZ, aBlock, UT.Code.bind4(aMetaData), 3);}
	public boolean setBlock(Block aBlock, int aMetaData, int aFlags) {return mWorld.setBlock(mX, mY, mZ, aBlock, UT.Code.bind4(aMetaData), aFlags);}
	public boolean setMetaData(int aMetaData) {return mWorld.setBlockMetadataWithNotify(mX, mY, mZ, UT.Code.bind4(aMetaData), 3);}
	public boolean setMetaData(int aMetaData, int aFlags) {return mWorld.setBlockMetadataWithNotify(mX, mY, mZ, UT.Code.bind4(aMetaData), aFlags);}
	
	public boolean equalCoords(DelegatorTileEntity aOther) {return aOther.mX == mX && aOther.mY == mY && aOther.mZ == mZ;}
	public boolean equalSideAndCoords(DelegatorTileEntity aOther) {return aOther.mSideOfTileEntity == mSideOfTileEntity && equalCoords(aOther);}
	public boolean equalSideWorldAndCoords(DelegatorTileEntity aOther) {return aOther.mWorld == mWorld && equalSideAndCoords(aOther);}
	public boolean equalSideTileEntityAndCoords(DelegatorTileEntity aOther) {return aOther.mTileEntity == mTileEntity && equalSideAndCoords(aOther);}
	
	@Override public World getWorld() {return mWorld;}
	@Override public int getX() {return mX;}
	@Override public int getY() {return mY;}
	@Override public int getZ() {return mZ;}
	@Override public int getOffsetX(byte aSide) {return mX + OFFSETS_X[aSide];}
    @Override public int getOffsetY(byte aSide) {return mY + OFFSETS_Y[aSide];}
    @Override public int getOffsetZ(byte aSide) {return mZ + OFFSETS_Z[aSide];}
	@Override public int getOffsetX(byte aSide, int aMultiplier) {return mX + OFFSETS_X[aSide] * aMultiplier;}
    @Override public int getOffsetY(byte aSide, int aMultiplier) {return mY + OFFSETS_Y[aSide] * aMultiplier;}
    @Override public int getOffsetZ(byte aSide, int aMultiplier) {return mZ + OFFSETS_Z[aSide] * aMultiplier;}
	@Override public ChunkCoordinates getCoords() {return new ChunkCoordinates(mX, mY, mZ);}
	@Override public ChunkCoordinates getOffset(byte aSide, int aMultiplier) {return new ChunkCoordinates(getOffsetX(aSide, aMultiplier), getOffsetY(aSide, aMultiplier), getOffsetZ(aSide, aMultiplier));}
	@Override public boolean isServerSide() {return mWorld == null ? cpw.mods.fml.common.FMLCommonHandler.instance().getEffectiveSide().isServer() : !mWorld.isRemote;}
	@Override public boolean isClientSide() {return mWorld == null ? cpw.mods.fml.common.FMLCommonHandler.instance().getEffectiveSide().isClient() :  mWorld.isRemote;}
	@Override public int getRandomNumber(int aRange) {return (mWorld==null?RANDOM:mWorld.rand).nextInt(aRange);}
	@Override public TileEntity getTileEntity	(int aX, int aY, int aZ) {return mWorld==null?null:mWorld.getTileEntity(aX, aY, aZ);}
	@Override public Block getBlock				(int aX, int aY, int aZ) {return mWorld==null?NB:mWorld.getBlock(aX, aY, aZ);}
	@Override public byte getMetaData			(int aX, int aY, int aZ) {return mWorld==null?0:UT.Code.bind4(mWorld.getBlockMetadata(aX, aY, aZ));}
	@Override public byte getLightLevel			(int aX, int aY, int aZ) {return mWorld==null?0:UT.Code.bind4((long)mWorld.getLightBrightness(aX, aY, aZ)*15);}
	@Override public boolean getOpacity			(int aX, int aY, int aZ) {return mWorld!=null&&mWorld.getBlock(aX, aY, aZ).isOpaqueCube();}
	@Override public boolean getSky				(int aX, int aY, int aZ) {return mWorld==null||mWorld.canBlockSeeTheSky(aX, aY, aZ);}
	@Override public boolean getRain			(int aX, int aY, int aZ) {return mWorld==null||mWorld.getPrecipitationHeight(aX, aZ) <= aY;}
	@Override public boolean getAir				(int aX, int aY, int aZ) {return mWorld==null||mWorld.getBlock(aX, aY, aZ).isAir(mWorld, aX, aY, aZ);}
	@Override public BiomeGenBase getBiome		(int aX, int aZ) {return mWorld==null?null:mWorld.getBiomeGenForCoords(aX, aZ);}
	@Override public TileEntity getTileEntity	(ChunkCoordinates aCoords) {return mWorld==null?null:mWorld.getTileEntity(aCoords.posX, aCoords.posY, aCoords.posZ);}
	@Override public Block getBlock				(ChunkCoordinates aCoords) {return mWorld==null?NB:mWorld.getBlock(aCoords.posX, aCoords.posY, aCoords.posZ);}
	@Override public byte getMetaData			(ChunkCoordinates aCoords) {return mWorld==null?0:UT.Code.bind4(mWorld.getBlockMetadata(aCoords.posX, aCoords.posY, aCoords.posZ));}
	@Override public byte getLightLevel			(ChunkCoordinates aCoords) {return mWorld==null?0:UT.Code.bind4((long)mWorld.getLightBrightness(aCoords.posX, aCoords.posY, aCoords.posZ)*15);}
	@Override public boolean getOpacity			(ChunkCoordinates aCoords) {return mWorld!=null&&mWorld.getBlock(aCoords.posX, aCoords.posY, aCoords.posZ).isOpaqueCube();}
	@Override public boolean getSky				(ChunkCoordinates aCoords) {return mWorld==null||mWorld.canBlockSeeTheSky(aCoords.posX, aCoords.posY, aCoords.posZ);}
	@Override public boolean getRain			(ChunkCoordinates aCoords) {return mWorld==null||mWorld.getPrecipitationHeight(aCoords.posX, aCoords.posZ) <= aCoords.posY;}
	@Override public boolean getAir				(ChunkCoordinates aCoords) {return mWorld==null||mWorld.getBlock(aCoords.posX, aCoords.posY, aCoords.posZ).isAir(mWorld, aCoords.posX, aCoords.posY, aCoords.posZ);}
	@Override public BiomeGenBase getBiome		(ChunkCoordinates aCoords) {return mWorld==null?null:mWorld.getBiomeGenForCoords(aCoords.posX, aCoords.posZ);}
}